@@ -292,6 +292,25 @@ class BaseWebDriver(DriverAPI):
     def cookies(self):
         return self._cookie_manager
 
+    @property
+    def current_window(self):
+        """
+        Returns the handle of the current window.
+        """
+        return self.driver.current_window_handle
+
+    @property
+    def windows(self):
+        """
+        Returns the handles of all windows within the current session.
+        """
+        return self.driver.window_handles
+
+    def switch_to_window(self, window_name):
+        """
+        Switches focus to the specified window.
+        """
+        return self.driver.switch_to_window(window_name)
 
 class TypeIterator(object):
 
@@ -314,7 +333,7 @@ class WebDriverElement(ElementAPI):
         self.action_chains = ActionChains(parent.driver)
 
     def _get_value(self):
-        return self["value"] or self._element.text
+        return self['value'] or self._element.text
 
     def _set_value(self, value):
         if self._element.get_attribute('type') != 'file':
@@ -362,6 +381,14 @@ class WebDriverElement(ElementAPI):
     def visible(self):
         return self._element.is_displayed()
 
+    @property
+    def html(self):
+        return self['innerHTML']
+
+    @property
+    def outer_html(self):
+        return self['outerHTML']
+
     def find_by_css(self, selector, original_find=None, original_query=None):
         find_by = original_find or 'css'
         query = original_query or selector
@@ -390,8 +417,7 @@ class WebDriverElement(ElementAPI):
         return ElementList([self.__class__(element, self.parent) for element in elements], find_by='id', query=id)
 
     def has_class(self, class_name):
-        element_class_name = self._element.get_attribute('class')
-        return bool(re.search(r'(?:^|\s)' + re.escape(class_name) + r'(?:$|\s)', element_class_name))
+        return bool(re.search(r'(?:^|\s)' + re.escape(class_name) + r'(?:$|\s)', self['class']))
 
     def mouse_over(self):
         """
